# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....commons.types.page_token import PageToken
from ....commons.types.payer_plan_group_id import PayerPlanGroupId
from ....commons.types.sort_direction import SortDirection
from ....commons.types.source_of_payment_code import SourceOfPaymentCode
from ....payers.resources.v_3.types.payer_id import PayerId
from ....payers.resources.v_3.types.payer_uuid import PayerUuid
from .raw_client import AsyncRawV1Client, RawV1Client
from .types.mutable_payer_plan_group import MutablePayerPlanGroup
from .types.payer_plan_group import PayerPlanGroup
from .types.payer_plan_group_page import PayerPlanGroupPage
from .types.payer_plan_group_sort_field import PayerPlanGroupSortField

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1Client
        """
        return self._raw_client

    def get_multi(
        self,
        *,
        plan_group_name: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        payer_uuid: typing.Optional[typing.Union[PayerUuid, typing.Sequence[PayerUuid]]] = None,
        payer_id: typing.Optional[typing.Union[PayerId, typing.Sequence[PayerId]]] = None,
        plan_type: typing.Optional[typing.Union[SourceOfPaymentCode, typing.Sequence[SourceOfPaymentCode]]] = None,
        is_active: typing.Optional[bool] = None,
        payer_plan_group_id: typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]] = None,
        limit: typing.Optional[int] = None,
        sort_by_similarity: typing.Optional[str] = None,
        sort: typing.Optional[PayerPlanGroupSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayerPlanGroupPage:
        """
        Returns all payer plan groups matching filter criteria.

        Parameters
        ----------
        plan_group_name : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        payer_uuid : typing.Optional[typing.Union[PayerUuid, typing.Sequence[PayerUuid]]]

        payer_id : typing.Optional[typing.Union[PayerId, typing.Sequence[PayerId]]]

        plan_type : typing.Optional[typing.Union[SourceOfPaymentCode, typing.Sequence[SourceOfPaymentCode]]]

        is_active : typing.Optional[bool]

        payer_plan_group_id : typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]]

        limit : typing.Optional[int]
            Defaults to 100. Cannot exc

        sort_by_similarity : typing.Optional[str]
            If this property is passed, the results will be ordered by those that contain a payer_id, payer_name, plan_group_name, or
            payer_address most similar to the value passed. This will take precedence over the sort and sort_direction properties. This
            will always sort in order of most similar to least similar.

        sort : typing.Optional[PayerPlanGroupSortField]
            Defaults to plan_group_name. If sort_by_similarity is passed, that sort will takes precedence over this property.

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to ascending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerPlanGroupPage

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.payer_plan_groups.v_1.get_multi()
        """
        _response = self._raw_client.get_multi(
            plan_group_name=plan_group_name,
            payer_uuid=payer_uuid,
            payer_id=payer_id,
            plan_type=plan_type,
            is_active=is_active,
            payer_plan_group_id=payer_plan_group_id,
            limit=limit,
            sort_by_similarity=sort_by_similarity,
            sort=sort,
            sort_direction=sort_direction,
            page_token=page_token,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, payer_plan_group_id: PayerPlanGroupId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayerPlanGroup:
        """
        Return a plan group with a given ID.

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerPlanGroup

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.payer_plan_groups.v_1.get(
            payer_plan_group_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get(payer_plan_group_id, request_options=request_options)
        return _response.data

    def create(
        self, *, request: MutablePayerPlanGroup, request_options: typing.Optional[RequestOptions] = None
    ) -> PayerPlanGroup:
        """
        Create a payer plan group

        Parameters
        ----------
        request : MutablePayerPlanGroup

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerPlanGroup

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import SourceOfPaymentCode
        from candid.resources.payer_plan_groups.resources.v_1 import (
            MutablePayerPlanGroup,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.payer_plan_groups.v_1.create(
            request=MutablePayerPlanGroup(
                plan_group_name="plan_group_name",
                payer_uuid=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                plan_type=SourceOfPaymentCode.SELF_PAY,
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def update(
        self,
        payer_plan_group_id: PayerPlanGroupId,
        *,
        request: MutablePayerPlanGroup,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayerPlanGroup:
        """
        Update any of the fields on a payer plan group

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request : MutablePayerPlanGroup

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerPlanGroup

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import SourceOfPaymentCode
        from candid.resources.payer_plan_groups.resources.v_1 import (
            MutablePayerPlanGroup,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.payer_plan_groups.v_1.update(
            payer_plan_group_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=MutablePayerPlanGroup(
                plan_group_name="plan_group_name",
                payer_uuid=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                plan_type=SourceOfPaymentCode.SELF_PAY,
            ),
        )
        """
        _response = self._raw_client.update(payer_plan_group_id, request=request, request_options=request_options)
        return _response.data

    def deactivate(
        self, payer_plan_group_id: PayerPlanGroupId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayerPlanGroup:
        """
        Marks the payer plan group as deactivated

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerPlanGroup

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.payer_plan_groups.v_1.deactivate(
            payer_plan_group_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.deactivate(payer_plan_group_id, request_options=request_options)
        return _response.data


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1Client
        """
        return self._raw_client

    async def get_multi(
        self,
        *,
        plan_group_name: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        payer_uuid: typing.Optional[typing.Union[PayerUuid, typing.Sequence[PayerUuid]]] = None,
        payer_id: typing.Optional[typing.Union[PayerId, typing.Sequence[PayerId]]] = None,
        plan_type: typing.Optional[typing.Union[SourceOfPaymentCode, typing.Sequence[SourceOfPaymentCode]]] = None,
        is_active: typing.Optional[bool] = None,
        payer_plan_group_id: typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]] = None,
        limit: typing.Optional[int] = None,
        sort_by_similarity: typing.Optional[str] = None,
        sort: typing.Optional[PayerPlanGroupSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayerPlanGroupPage:
        """
        Returns all payer plan groups matching filter criteria.

        Parameters
        ----------
        plan_group_name : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        payer_uuid : typing.Optional[typing.Union[PayerUuid, typing.Sequence[PayerUuid]]]

        payer_id : typing.Optional[typing.Union[PayerId, typing.Sequence[PayerId]]]

        plan_type : typing.Optional[typing.Union[SourceOfPaymentCode, typing.Sequence[SourceOfPaymentCode]]]

        is_active : typing.Optional[bool]

        payer_plan_group_id : typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]]

        limit : typing.Optional[int]
            Defaults to 100. Cannot exc

        sort_by_similarity : typing.Optional[str]
            If this property is passed, the results will be ordered by those that contain a payer_id, payer_name, plan_group_name, or
            payer_address most similar to the value passed. This will take precedence over the sort and sort_direction properties. This
            will always sort in order of most similar to least similar.

        sort : typing.Optional[PayerPlanGroupSortField]
            Defaults to plan_group_name. If sort_by_similarity is passed, that sort will takes precedence over this property.

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to ascending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerPlanGroupPage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.payer_plan_groups.v_1.get_multi()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_multi(
            plan_group_name=plan_group_name,
            payer_uuid=payer_uuid,
            payer_id=payer_id,
            plan_type=plan_type,
            is_active=is_active,
            payer_plan_group_id=payer_plan_group_id,
            limit=limit,
            sort_by_similarity=sort_by_similarity,
            sort=sort,
            sort_direction=sort_direction,
            page_token=page_token,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, payer_plan_group_id: PayerPlanGroupId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayerPlanGroup:
        """
        Return a plan group with a given ID.

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerPlanGroup

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.payer_plan_groups.v_1.get(
                payer_plan_group_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(payer_plan_group_id, request_options=request_options)
        return _response.data

    async def create(
        self, *, request: MutablePayerPlanGroup, request_options: typing.Optional[RequestOptions] = None
    ) -> PayerPlanGroup:
        """
        Create a payer plan group

        Parameters
        ----------
        request : MutablePayerPlanGroup

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerPlanGroup

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import SourceOfPaymentCode
        from candid.resources.payer_plan_groups.resources.v_1 import (
            MutablePayerPlanGroup,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.payer_plan_groups.v_1.create(
                request=MutablePayerPlanGroup(
                    plan_group_name="plan_group_name",
                    payer_uuid=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    plan_type=SourceOfPaymentCode.SELF_PAY,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def update(
        self,
        payer_plan_group_id: PayerPlanGroupId,
        *,
        request: MutablePayerPlanGroup,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayerPlanGroup:
        """
        Update any of the fields on a payer plan group

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request : MutablePayerPlanGroup

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerPlanGroup

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import SourceOfPaymentCode
        from candid.resources.payer_plan_groups.resources.v_1 import (
            MutablePayerPlanGroup,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.payer_plan_groups.v_1.update(
                payer_plan_group_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=MutablePayerPlanGroup(
                    plan_group_name="plan_group_name",
                    payer_uuid=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    plan_type=SourceOfPaymentCode.SELF_PAY,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(payer_plan_group_id, request=request, request_options=request_options)
        return _response.data

    async def deactivate(
        self, payer_plan_group_id: PayerPlanGroupId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayerPlanGroup:
        """
        Marks the payer plan group as deactivated

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerPlanGroup

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.payer_plan_groups.v_1.deactivate(
                payer_plan_group_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.deactivate(payer_plan_group_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .expanded_member_info import ExpandedMemberInfo
from .plan_date import PlanDate


class PlanMetadata(UniversalBaseModel):
    payer_name: typing.Optional[str] = None
    insurance_type: typing.Optional[str] = None
    insurance_type_code: typing.Optional[str] = None
    plan_name: typing.Optional[str] = None
    member_id: typing.Optional[str] = None
    group_number: typing.Optional[str] = None
    start_date: typing.Optional[dt.date] = None
    end_date: typing.Optional[dt.date] = None
    plan_dates: typing.Optional[typing.List[PlanDate]] = None
    subscriber: typing.Optional[ExpandedMemberInfo] = None
    dependent: typing.Optional[ExpandedMemberInfo] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ...commons.types.facility_type_code import FacilityTypeCode
from ...commons.types.state import State
from ...encounter_providers.resources.v_2.types.billing_provider import BillingProvider
from ...encounter_providers.resources.v_2.types.rendering_provider import RenderingProvider
from ...encounters.resources.v_4.types.encounter_submission_expectation import EncounterSubmissionExpectation
from ...health_care_code_information.resources.v_1.types.health_care_code_information_create import (
    HealthCareCodeInformationCreate,
)
from ...service_lines.resources.v_2.types.universal_service_line_create import UniversalServiceLineCreate
from ...x_12.resources.v_1.types.patient_discharge_status_code import PatientDischargeStatusCode
from ...x_12.resources.v_1.types.point_of_origin_for_admission_or_visit_code import PointOfOriginForAdmissionOrVisitCode
from ...x_12.resources.v_1.types.type_of_admission_or_visit_code import TypeOfAdmissionOrVisitCode
from ...x_12.resources.v_1.types.type_of_bill_composite_update import TypeOfBillCompositeUpdate
from .universal_encounter_create_base import UniversalEncounterCreateBase


class UniversalEncounterCreate(UniversalEncounterCreateBase):
    billing_provider: BillingProvider = pydantic.Field()
    """
    The billing provider is the provider or business entity submitting the claim. Billing provider may be, but is not necessarily, the same person/NPI as the rendering provider. From a payer's perspective, this represents the person or entity being reimbursed. When a contract exists with the target payer, the billing provider should be the entity contracted with the payer. In some circumstances, this will be an individual provider. In that case, submit that provider's NPI and the tax ID (TIN) that the provider gave to the payer during contracting. In other cases, the billing entity will be a medical group. If so, submit the group NPI and the group's tax ID. Box 33 on the CMS-1500 claim or Form Locator 1 on a UB-04 claim form.
    """

    place_of_service_code: typing.Optional[FacilityTypeCode] = pydantic.Field(default=None)
    """
    Box 24B on the CMS-1500 claim form. 837p Loop2300, CLM-05-1. This box is not used on a UB-04 or 837i claim form. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).
    """

    rendering_provider: typing.Optional[RenderingProvider] = pydantic.Field(default=None)
    """
    The rendering provider is the practitioner -- physician, nurse practitioner, etc. -- performing the service.
    For telehealth services, the rendering provider performs the visit, asynchronous communication, or other service. The rendering provider address should generally be the same as the service facility address.
    """

    service_lines: typing.Optional[typing.List[UniversalServiceLineCreate]] = None
    health_care_code_information: typing.Optional[HealthCareCodeInformationCreate] = pydantic.Field(default=None)
    """
    The health care code information associated with this encounter, which includes things such as diagnoses, vitals, procedures, occurrences, reason for visit, and numerous other code related fields.
    """

    submission_expectation: EncounterSubmissionExpectation = pydantic.Field()
    """
    Describes the currently expected target form for this encounter.  This can change later, but it orients this encounter in the pipeline and effects reporting.
    """

    attending_provider: typing.Optional[RenderingProvider] = pydantic.Field(default=None)
    """
    837i NM1 2500 variant for Loop ID-2310.  Used to indicate the individual whom has overall responsibility for the patient in institutional claims processing.
    """

    admission_hour: typing.Optional[int] = pydantic.Field(default=None)
    """
    837i Loop 2300 DTP-03
    Extension of the admission date with hour (0-23 integer) details.
    """

    admission_type_code: typing.Optional[TypeOfAdmissionOrVisitCode] = pydantic.Field(default=None)
    """
    837i Loop 2300 CL1-01 Code used to indicate the priority of an admission or visit. Equivalent to Form Locator 14 Priority of Admission on a UB-04 claim, not used on CMS-1500 claim forms.
    """

    admission_source_code: typing.Optional[PointOfOriginForAdmissionOrVisitCode] = pydantic.Field(default=None)
    """
    837i Loop 2300 CLI1-02 Code used to indicate the conditions under which an admission occurs. Equivalent to Form Locator 15 Point of Origin on a UB-04 claim, not used on CMS-1500 claim forms.
    """

    discharge_hour: typing.Optional[int] = pydantic.Field(default=None)
    """
    837i Loop 2300 DTP-03
    Extension of the discharge date with hour (0-23 integer) details.
    """

    discharge_status: typing.Optional[PatientDischargeStatusCode] = pydantic.Field(default=None)
    """
    837i CL1-03 or Form Locator 17 on a UB-04 claim form. This is a required field on UB-04 claims.
    Code indicating patient status as of the "statement covers through date" .
    """

    operating_provider: typing.Optional[RenderingProvider] = pydantic.Field(default=None)
    """
    837i NM1 2500 variant for Loop ID-2310.  Used to indicate the individual whom has primary responsibility for surgical procedures in institutional claims processing.
    """

    other_operating_provider: typing.Optional[RenderingProvider] = pydantic.Field(default=None)
    """
    837i NM1 2500 variant for Loop ID-2310.  Used to indicate the individual whom has secondary responsibility for surgical procedures in institutional claims processing.  Only used when operating_provider is also set.
    """

    type_of_bill: typing.Optional[TypeOfBillCompositeUpdate] = pydantic.Field(default=None)
    """
    Four digit code used in institutional forms to indicate the type of bill (e.g., hospital inpatient, hospital outpatient). First digit is a leading 0, followed by the type_of_facility, type_of_care, then frequency_code. Professional forms are not required to submit this attribute. You may send the 4 digit code via raw_code, or each individual digit separately via composite_codes.
    """

    accident_state_or_province_code: typing.Optional[State] = pydantic.Field(default=None)
    """
    837i-REF1000 -- an optional state indicating where an accident related to the encounter occurred.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
import uuid

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .contract_base import ContractBase
from .contract_type import ContractType
from .contracting_provider_id import ContractingProviderId


class ContractCreate(ContractBase):
    contract_type: ContractType = pydantic.Field()
    """
    The type of contract
    """

    contracting_provider_id: ContractingProviderId = pydantic.Field()
    """
    The UUID of the provider under agreement to the contract
    """

    payer_uuid: uuid.UUID = pydantic.Field()
    """
    The UUID of the insurance company under agreement to the contract
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .intake_question_id import IntakeQuestionId
from .intake_response_and_follow_ups import IntakeResponseAndFollowUps


class IntakeQuestionOptional(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.encounters.resources.v_4 import (
        IntakeFollowUp,
        IntakeQuestionOptional,
        IntakeResponseAndFollowUps,
    )

    IntakeQuestionOptional(
        id="6E7FBCE4-A8EA-46D0-A8D8-FF83CA3BB176",
        text="Do you have any allergies?",
        responses=[
            IntakeResponseAndFollowUps(
                response="No allergies",
                follow_ups=[
                    IntakeFollowUp(
                        id="4F3D57F9-AC94-49D6-87E4-E804B709917A",
                        text="Do you have any allergies?",
                        response="No allergies",
                    )
                ],
            )
        ],
    )
    """

    id: typing.Optional[IntakeQuestionId] = None
    text: typing.Optional[str] = None
    responses: typing.Optional[typing.List[IntakeResponseAndFollowUps]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class ResponsiblePartyType(enum.StrEnum):
    INSURANCE_PAY = "INSURANCE_PAY"
    SELF_PAY = "SELF_PAY"
    UNKNOWN = "UNKNOWN"
    _UNKNOWN = "__RESPONSIBLEPARTYTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ResponsiblePartyType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        insurance_pay: typing.Callable[[], T_Result],
        self_pay: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ResponsiblePartyType.INSURANCE_PAY:
            return insurance_pay()
        if self is ResponsiblePartyType.SELF_PAY:
            return self_pay()
        if self is ResponsiblePartyType.UNKNOWN:
            return unknown()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .diagnosis_related_group_code_qualifier import DiagnosisRelatedGroupCodeQualifier
from .health_care_code_information import HealthCareCodeInformation
from .msdrg_code import MsdrgCode


class DiagnosisRelatedGroup(HealthCareCodeInformation):
    """
    This corresponds to DR code qualifier values.
    """

    diagnosis_related_group_code_qualifier: DiagnosisRelatedGroupCodeQualifier
    diagnosis_related_group_code: MsdrgCode

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.rate_id import RateId
from .dimensions import Dimensions
from .rate_entry import RateEntry


class RateUpload_NewRate(UniversalBaseModel):
    type: typing.Literal["new_rate"] = "new_rate"
    dimensions: Dimensions
    entries: typing.List[RateEntry]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class RateUpload_NewVersion(UniversalBaseModel):
    type: typing.Literal["new_version"] = "new_version"
    rate_id: RateId
    previous_version: int
    entries: typing.List[RateEntry]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


RateUpload = typing_extensions.Annotated[
    typing.Union[RateUpload_NewRate, RateUpload_NewVersion], pydantic.Field(discriminator="type")
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawAuthClient, RawAuthClient

if typing.TYPE_CHECKING:
    from .resources.default.client import AsyncDefaultClient, DefaultClient


class AuthClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAuthClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._default: typing.Optional[DefaultClient] = None

    @property
    def with_raw_response(self) -> RawAuthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAuthClient
        """
        return self._raw_client

    @property
    def default(self):
        if self._default is None:
            from .resources.default.client import DefaultClient  # noqa: E402

            self._default = DefaultClient(client_wrapper=self._client_wrapper)
        return self._default


class AsyncAuthClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAuthClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._default: typing.Optional[AsyncDefaultClient] = None

    @property
    def with_raw_response(self) -> AsyncRawAuthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAuthClient
        """
        return self._raw_client

    @property
    def default(self):
        if self._default is None:
            from .resources.default.client import AsyncDefaultClient  # noqa: E402

            self._default = AsyncDefaultClient(client_wrapper=self._client_wrapper)
        return self._default



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class PayerCategory(enum.StrEnum):
    BCBS = "BCBS"
    AETNA_AFFILIATED = "AETNA_AFFILIATED"
    CIGNA_AFFILIATED = "CIGNA_AFFILIATED"
    UNITED_AFFILIATED = "UNITED_AFFILIATED"
    MEDICARE = "MEDICARE"
    NON_TRADITIONAL_MEDICARE = "NON_TRADITIONAL_MEDICARE"
    MEDICAID = "MEDICAID"
    HOSPICE = "HOSPICE"
    SNF = "SNF"
    TRICARE = "TRICARE"
    WORKERS_COMP = "WORKERS_COMP"
    OTHER_GOVERNMENT = "OTHER_GOVERNMENT"
    AUTO_TPL = "AUTO_TPL"
    _UNKNOWN = "__PAYERCATEGORY_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "PayerCategory":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        bcbs: typing.Callable[[], T_Result],
        aetna_affiliated: typing.Callable[[], T_Result],
        cigna_affiliated: typing.Callable[[], T_Result],
        united_affiliated: typing.Callable[[], T_Result],
        medicare: typing.Callable[[], T_Result],
        non_traditional_medicare: typing.Callable[[], T_Result],
        medicaid: typing.Callable[[], T_Result],
        hospice: typing.Callable[[], T_Result],
        snf: typing.Callable[[], T_Result],
        tricare: typing.Callable[[], T_Result],
        workers_comp: typing.Callable[[], T_Result],
        other_government: typing.Callable[[], T_Result],
        auto_tpl: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is PayerCategory.BCBS:
            return bcbs()
        if self is PayerCategory.AETNA_AFFILIATED:
            return aetna_affiliated()
        if self is PayerCategory.CIGNA_AFFILIATED:
            return cigna_affiliated()
        if self is PayerCategory.UNITED_AFFILIATED:
            return united_affiliated()
        if self is PayerCategory.MEDICARE:
            return medicare()
        if self is PayerCategory.NON_TRADITIONAL_MEDICARE:
            return non_traditional_medicare()
        if self is PayerCategory.MEDICAID:
            return medicaid()
        if self is PayerCategory.HOSPICE:
            return hospice()
        if self is PayerCategory.SNF:
            return snf()
        if self is PayerCategory.TRICARE:
            return tricare()
        if self is PayerCategory.WORKERS_COMP:
            return workers_comp()
        if self is PayerCategory.OTHER_GOVERNMENT:
            return other_government()
        if self is PayerCategory.AUTO_TPL:
            return auto_tpl()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core import enum

T_Result = typing.TypeVar("T_Result")


class YesNoIndicator(enum.StrEnum):
    YES = "YES"
    NO = "NO"
    UNKNOWN = "UNKNOWN"
    NOT_APPLICABLE = "NOT_APPLICABLE"
    _UNKNOWN = "__YESNOINDICATOR_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "YesNoIndicator":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        yes: typing.Callable[[], T_Result],
        no: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        not_applicable: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is YesNoIndicator.YES:
            return yes()
        if self is YesNoIndicator.NO:
            return no()
        if self is YesNoIndicator.UNKNOWN:
            return unknown()
        if self is YesNoIndicator.NOT_APPLICABLE:
            return not_applicable()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class TypeOfAdmissionOrVisitCode(enum.StrEnum):
    C_1 = "1"
    """
    EMERGENCY - As a result of severe, life threatening or potentially disabling conditions, the patient requires immediate medical intervention.
    """

    C_2 = "2"
    """
    URGENT - The patient requires immediate attention for the care and treatment of a physical or mental disorder.
    """

    C_3 = "3"
    """
    ELECTIVE - The patient's condition permits adequate time to schedule the services.
    """

    C_4 = "4"
    """
    NEWBORN - Use of this code necessitates the use of special Point of Origin Codes (See Form Locator 15 page 5 of 9).
    """

    C_5 = "5"
    """
    TRAUMA - Visit to a trauma center/hospital as licensed or designated by the state or local government authority authorized to do so, or as verified by the American College of Surgeons and involving a trauma activation. (Use Revenue Code 068x capture trauma activation charges involving pre-hospital notification.)
    """

    C_9 = "9"
    """
    INFORMATION NOT AVAILABLE - Information not available.
    """

    _UNKNOWN = "__TYPEOFADMISSIONORVISITCODE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "TypeOfAdmissionOrVisitCode":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        c_1: typing.Callable[[], T_Result],
        c_2: typing.Callable[[], T_Result],
        c_3: typing.Callable[[], T_Result],
        c_4: typing.Callable[[], T_Result],
        c_5: typing.Callable[[], T_Result],
        c_9: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is TypeOfAdmissionOrVisitCode.C_1:
            return c_1()
        if self is TypeOfAdmissionOrVisitCode.C_2:
            return c_2()
        if self is TypeOfAdmissionOrVisitCode.C_3:
            return c_3()
        if self is TypeOfAdmissionOrVisitCode.C_4:
            return c_4()
        if self is TypeOfAdmissionOrVisitCode.C_5:
            return c_5()
        if self is TypeOfAdmissionOrVisitCode.C_9:
            return c_9()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing
import uuid

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....commons.types.regions import Regions
from .....organization_providers.resources.v_3.types.organization_provider_v_2 import OrganizationProviderV2
from .base_credentialing_span import BaseCredentialingSpan
from .provider_credentialing_span_id import ProviderCredentialingSpanId


class ProviderCredentialingSpan(BaseCredentialingSpan):
    provider_credentialing_span_id: ProviderCredentialingSpanId
    rendering_provider: OrganizationProviderV2 = pydantic.Field()
    """
    The rendering provider covered by the credentialing span.
    """

    regions: Regions = pydantic.Field()
    """
    The states covered by the credentialing span. A span may be national and cover all states.
    """

    medallion_payer_enrollment_id: typing.Optional[uuid.UUID] = pydantic.Field(default=None)
    """
    Provider ID for the related medallion payer enrollment.
    """

    source: str = pydantic.Field()
    """
    Source of the credentialing span.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....coverages.resources.v_1.types.coverage import Coverage
from .patient import Patient


class PatientCoverageSnapshot(UniversalBaseModel):
    patient: Patient
    primary_coverage: typing.Optional[Coverage] = None
    secondary_coverage: typing.Optional[Coverage] = None
    tertiary_coverage: typing.Optional[Coverage] = None
    coverages_for_related_causes: typing.List[Coverage]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.address import Address
from .....common.types.human_name import HumanName
from .....common.types.sex import Sex


class Subscriber(UniversalBaseModel):
    name: HumanName
    date_of_birth: typing.Optional[dt.date] = None
    biological_sex: Sex
    address: typing.Optional[Address] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawCredentialingClient, RawCredentialingClient

if typing.TYPE_CHECKING:
    from .resources.v_2.client import AsyncV2Client, V2Client


class CredentialingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCredentialingClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._v_2: typing.Optional[V2Client] = None

    @property
    def with_raw_response(self) -> RawCredentialingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCredentialingClient
        """
        return self._raw_client

    @property
    def v_2(self):
        if self._v_2 is None:
            from .resources.v_2.client import V2Client  # noqa: E402

            self._v_2 = V2Client(client_wrapper=self._client_wrapper)
        return self._v_2


class AsyncCredentialingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCredentialingClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._v_2: typing.Optional[AsyncV2Client] = None

    @property
    def with_raw_response(self) -> AsyncRawCredentialingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCredentialingClient
        """
        return self._raw_client

    @property
    def v_2(self):
        if self._v_2 is None:
            from .resources.v_2.client import AsyncV2Client  # noqa: E402

            self._v_2 = AsyncV2Client(client_wrapper=self._client_wrapper)
        return self._v_2



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class PatientPaymentAllocation(UniversalBaseModel):
    source_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Payment identifier from the source system (e.g., patient portal, manual entry). Note multiple allocations may exist for a single payment and
    thus may share the same source_id.
    """

    amount_cents: int = pydantic.Field()
    """
    The amount of the patient payment in cents.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .carc import Carc
    from .claim_adjustment_reason_code import ClaimAdjustmentReasonCode
    from .patient_discharge_status_code import PatientDischargeStatusCode
    from .point_of_origin_for_admission_or_visit_code import PointOfOriginForAdmissionOrVisitCode
    from .rarc import Rarc
    from .remittance_advice_remark_code import RemittanceAdviceRemarkCode
    from .type_of_admission_or_visit_code import TypeOfAdmissionOrVisitCode
    from .type_of_bill_composite import TypeOfBillComposite
    from .type_of_bill_composite_base import TypeOfBillCompositeBase
    from .type_of_bill_composite_update import (
        TypeOfBillCompositeUpdate,
        TypeOfBillCompositeUpdate_CompositeCodes,
        TypeOfBillCompositeUpdate_RawCode,
    )
    from .type_of_bill_frequency_code import TypeOfBillFrequencyCode
    from .type_of_care_code import TypeOfCareCode
    from .type_of_facility_code import TypeOfFacilityCode
_dynamic_imports: typing.Dict[str, str] = {
    "Carc": ".carc",
    "ClaimAdjustmentReasonCode": ".claim_adjustment_reason_code",
    "PatientDischargeStatusCode": ".patient_discharge_status_code",
    "PointOfOriginForAdmissionOrVisitCode": ".point_of_origin_for_admission_or_visit_code",
    "Rarc": ".rarc",
    "RemittanceAdviceRemarkCode": ".remittance_advice_remark_code",
    "TypeOfAdmissionOrVisitCode": ".type_of_admission_or_visit_code",
    "TypeOfBillComposite": ".type_of_bill_composite",
    "TypeOfBillCompositeBase": ".type_of_bill_composite_base",
    "TypeOfBillCompositeUpdate": ".type_of_bill_composite_update",
    "TypeOfBillCompositeUpdate_CompositeCodes": ".type_of_bill_composite_update",
    "TypeOfBillCompositeUpdate_RawCode": ".type_of_bill_composite_update",
    "TypeOfBillFrequencyCode": ".type_of_bill_frequency_code",
    "TypeOfCareCode": ".type_of_care_code",
    "TypeOfFacilityCode": ".type_of_facility_code",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Carc",
    "ClaimAdjustmentReasonCode",
    "PatientDischargeStatusCode",
    "PointOfOriginForAdmissionOrVisitCode",
    "Rarc",
    "RemittanceAdviceRemarkCode",
    "TypeOfAdmissionOrVisitCode",
    "TypeOfBillComposite",
    "TypeOfBillCompositeBase",
    "TypeOfBillCompositeUpdate",
    "TypeOfBillCompositeUpdate_CompositeCodes",
    "TypeOfBillCompositeUpdate_RawCode",
    "TypeOfBillFrequencyCode",
    "TypeOfCareCode",
    "TypeOfFacilityCode",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .organization_service_facility import OrganizationServiceFacility
    from .organization_service_facility_create import OrganizationServiceFacilityCreate
    from .organization_service_facility_id import OrganizationServiceFacilityId
    from .organization_service_facility_page import OrganizationServiceFacilityPage
    from .organization_service_facility_update import OrganizationServiceFacilityUpdate
    from .service_facility_mode import ServiceFacilityMode
    from .service_facility_operational_status import ServiceFacilityOperationalStatus
    from .service_facility_physical_type import ServiceFacilityPhysicalType
    from .service_facility_status import ServiceFacilityStatus
    from .service_facility_type import ServiceFacilityType
_dynamic_imports: typing.Dict[str, str] = {
    "OrganizationServiceFacility": ".organization_service_facility",
    "OrganizationServiceFacilityCreate": ".organization_service_facility_create",
    "OrganizationServiceFacilityId": ".organization_service_facility_id",
    "OrganizationServiceFacilityPage": ".organization_service_facility_page",
    "OrganizationServiceFacilityUpdate": ".organization_service_facility_update",
    "ServiceFacilityMode": ".service_facility_mode",
    "ServiceFacilityOperationalStatus": ".service_facility_operational_status",
    "ServiceFacilityPhysicalType": ".service_facility_physical_type",
    "ServiceFacilityStatus": ".service_facility_status",
    "ServiceFacilityType": ".service_facility_type",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "OrganizationServiceFacility",
    "OrganizationServiceFacilityCreate",
    "OrganizationServiceFacilityId",
    "OrganizationServiceFacilityPage",
    "OrganizationServiceFacilityUpdate",
    "ServiceFacilityMode",
    "ServiceFacilityOperationalStatus",
    "ServiceFacilityPhysicalType",
    "ServiceFacilityStatus",
    "ServiceFacilityType",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        BillableStatusType,
        CashPayPayerErrorMessage,
        ClaimSupplementalInformation,
        ClaimSupplementalInformationOptional,
        ClinicalNote,
        ClinicalNoteCategory,
        ClinicalNoteCategoryCreate,
        ClinicalNoteCategoryCreateOptional,
        ClinicalNoteOptional,
        CodingAttributionType,
        Encounter,
        EncounterBase,
        EncounterCreate,
        EncounterCreateFromPreEncounter,
        EncounterDeepOptional,
        EncounterExternalIdUniquenessErrorType,
        EncounterGuarantorMissingContactInfoErrorType,
        EncounterOptional,
        EncounterOwnerOfNextActionType,
        EncounterPage,
        EncounterPatientControlNumberUniquenessErrorType,
        EncounterRenderingOrAttendingProviderRequiredError,
        EncounterSortOptions,
        EncounterSubmissionExpectation,
        EncounterSubmissionOriginType,
        EncounterUpdate,
        EpsdtReferral,
        EpsdtReferralOptional,
        IntakeFollowUp,
        IntakeFollowUpId,
        IntakeQuestion,
        IntakeQuestionId,
        IntakeQuestionOptional,
        IntakeResponseAndFollowUps,
        Intervention,
        InterventionCategory,
        InterventionOptional,
        InvalidTagNamesErrorType,
        KeyDoesNotExistError,
        Lab,
        LabCodeType,
        LabOptional,
        Medication,
        MedicationOptional,
        MultipleInstancesForSchemaError,
        NoteCategory,
        PatientHistoryCategory,
        PatientHistoryCategoryEnum,
        PatientHistoryCategoryOptional,
        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
        PriorAuthorizationNumber,
        ResponsiblePartyType,
        RxCui,
        SchemaDoesNotExistError,
        SchemaInstanceValidationError,
        SchemaInstanceValidationError_KeyDoesNotExist,
        SchemaInstanceValidationError_MultipleInstancesForSchema,
        SchemaInstanceValidationError_SchemaDoesNotExist,
        SchemaInstanceValidationError_SchemaUnauthorizedAccess,
        SchemaInstanceValidationError_ValueDoesNotMatchKeyType,
        SchemaInstanceValidationFailure,
        SchemaUnauthorizedAccessError,
        ServiceAuthorizationExceptionCode,
        SynchronicityType,
        ValueDoesNotMatchKeyTypeError,
        Vitals,
        VitalsUpdate,
    )
    from .errors import (
        CashPayPayerError,
        EncounterExternalIdUniquenessError,
        EncounterGuarantorMissingContactInfoError,
        EncounterPatientControlNumberUniquenessError,
        EncounterRenderingOrAttendingProviderRequired,
        InvalidTagNamesError,
        PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError,
        SchemaInstanceValidationHttpFailure,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "BillableStatusType": ".types",
    "CashPayPayerError": ".errors",
    "CashPayPayerErrorMessage": ".types",
    "ClaimSupplementalInformation": ".types",
    "ClaimSupplementalInformationOptional": ".types",
    "ClinicalNote": ".types",
    "ClinicalNoteCategory": ".types",
    "ClinicalNoteCategoryCreate": ".types",
    "ClinicalNoteCategoryCreateOptional": ".types",
    "ClinicalNoteOptional": ".types",
    "CodingAttributionType": ".types",
    "Encounter": ".types",
    "EncounterBase": ".types",
    "EncounterCreate": ".types",
    "EncounterCreateFromPreEncounter": ".types",
    "EncounterDeepOptional": ".types",
    "EncounterExternalIdUniquenessError": ".errors",
    "EncounterExternalIdUniquenessErrorType": ".types",
    "EncounterGuarantorMissingContactInfoError": ".errors",
    "EncounterGuarantorMissingContactInfoErrorType": ".types",
    "EncounterOptional": ".types",
    "EncounterOwnerOfNextActionType": ".types",
    "EncounterPage": ".types",
    "EncounterPatientControlNumberUniquenessError": ".errors",
    "EncounterPatientControlNumberUniquenessErrorType": ".types",
    "EncounterRenderingOrAttendingProviderRequired": ".errors",
    "EncounterRenderingOrAttendingProviderRequiredError": ".types",
    "EncounterSortOptions": ".types",
    "EncounterSubmissionExpectation": ".types",
    "EncounterSubmissionOriginType": ".types",
    "EncounterUpdate": ".types",
    "EpsdtReferral": ".types",
    "EpsdtReferralOptional": ".types",
    "IntakeFollowUp": ".types",
    "IntakeFollowUpId": ".types",
    "IntakeQuestion": ".types",
    "IntakeQuestionId": ".types",
    "IntakeQuestionOptional": ".types",
    "IntakeResponseAndFollowUps": ".types",
    "Intervention": ".types",
    "InterventionCategory": ".types",
    "InterventionOptional": ".types",
    "InvalidTagNamesError": ".errors",
    "InvalidTagNamesErrorType": ".types",
    "KeyDoesNotExistError": ".types",
    "Lab": ".types",
    "LabCodeType": ".types",
    "LabOptional": ".types",
    "Medication": ".types",
    "MedicationOptional": ".types",
    "MultipleInstancesForSchemaError": ".types",
    "NoteCategory": ".types",
    "PatientHistoryCategory": ".types",
    "PatientHistoryCategoryEnum": ".types",
    "PatientHistoryCategoryOptional": ".types",
    "PayerPlanGroupPayerDoesNotMatchInsuranceCardError": ".types",
    "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError": ".errors",
    "PriorAuthorizationNumber": ".types",
    "ResponsiblePartyType": ".types",
    "RxCui": ".types",
    "SchemaDoesNotExistError": ".types",
    "SchemaInstanceValidationError": ".types",
    "SchemaInstanceValidationError_KeyDoesNotExist": ".types",
    "SchemaInstanceValidationError_MultipleInstancesForSchema": ".types",
    "SchemaInstanceValidationError_SchemaDoesNotExist": ".types",
    "SchemaInstanceValidationError_SchemaUnauthorizedAccess": ".types",
    "SchemaInstanceValidationError_ValueDoesNotMatchKeyType": ".types",
    "SchemaInstanceValidationFailure": ".types",
    "SchemaInstanceValidationHttpFailure": ".errors",
    "SchemaUnauthorizedAccessError": ".types",
    "ServiceAuthorizationExceptionCode": ".types",
    "SynchronicityType": ".types",
    "ValueDoesNotMatchKeyTypeError": ".types",
    "Vitals": ".types",
    "VitalsUpdate": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BillableStatusType",
    "CashPayPayerError",
    "CashPayPayerErrorMessage",
    "ClaimSupplementalInformation",
    "ClaimSupplementalInformationOptional",
    "ClinicalNote",
    "ClinicalNoteCategory",
    "ClinicalNoteCategoryCreate",
    "ClinicalNoteCategoryCreateOptional",
    "ClinicalNoteOptional",
    "CodingAttributionType",
    "Encounter",
    "EncounterBase",
    "EncounterCreate",
    "EncounterCreateFromPreEncounter",
    "EncounterDeepOptional",
    "EncounterExternalIdUniquenessError",
    "EncounterExternalIdUniquenessErrorType",
    "EncounterGuarantorMissingContactInfoError",
    "EncounterGuarantorMissingContactInfoErrorType",
    "EncounterOptional",
    "EncounterOwnerOfNextActionType",
    "EncounterPage",
    "EncounterPatientControlNumberUniquenessError",
    "EncounterPatientControlNumberUniquenessErrorType",
    "EncounterRenderingOrAttendingProviderRequired",
    "EncounterRenderingOrAttendingProviderRequiredError",
    "EncounterSortOptions",
    "EncounterSubmissionExpectation",
    "EncounterSubmissionOriginType",
    "EncounterUpdate",
    "EpsdtReferral",
    "EpsdtReferralOptional",
    "IntakeFollowUp",
    "IntakeFollowUpId",
    "IntakeQuestion",
    "IntakeQuestionId",
    "IntakeQuestionOptional",
    "IntakeResponseAndFollowUps",
    "Intervention",
    "InterventionCategory",
    "InterventionOptional",
    "InvalidTagNamesError",
    "InvalidTagNamesErrorType",
    "KeyDoesNotExistError",
    "Lab",
    "LabCodeType",
    "LabOptional",
    "Medication",
    "MedicationOptional",
    "MultipleInstancesForSchemaError",
    "NoteCategory",
    "PatientHistoryCategory",
    "PatientHistoryCategoryEnum",
    "PatientHistoryCategoryOptional",
    "PayerPlanGroupPayerDoesNotMatchInsuranceCardError",
    "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError",
    "PriorAuthorizationNumber",
    "ResponsiblePartyType",
    "RxCui",
    "SchemaDoesNotExistError",
    "SchemaInstanceValidationError",
    "SchemaInstanceValidationError_KeyDoesNotExist",
    "SchemaInstanceValidationError_MultipleInstancesForSchema",
    "SchemaInstanceValidationError_SchemaDoesNotExist",
    "SchemaInstanceValidationError_SchemaUnauthorizedAccess",
    "SchemaInstanceValidationError_ValueDoesNotMatchKeyType",
    "SchemaInstanceValidationFailure",
    "SchemaInstanceValidationHttpFailure",
    "SchemaUnauthorizedAccessError",
    "ServiceAuthorizationExceptionCode",
    "SynchronicityType",
    "ValueDoesNotMatchKeyTypeError",
    "Vitals",
    "VitalsUpdate",
]



# This file was auto-generated by Fern from our API Definition.

import typing
import uuid

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.charge_capture_claim_creation_id import ChargeCaptureClaimCreationId
from .....commons.types.charge_capture_id import ChargeCaptureId


class ChargeCaptureError(UniversalBaseModel):
    id: uuid.UUID
    charge_capture_id: typing.Optional[ChargeCaptureId] = pydantic.Field(default=None)
    """
    The underlying Charge Capture that this error object references.
    The Charge Capture referenced will be a part of the Claim Creation tied to this error.
    Errors may also refer to all charge_captures present in a Claim Creation, in which case this field will be null.
    """

    message: str = pydantic.Field()
    """
    A human readable error explaining why this charge capture Claim Creation failed to create a claim.
    """

    resolved: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether or not the error has been resolved as part of user workflows
    """

    resolved_by: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string denoting who resolved the error within candid's system.
    """

    resolution_reason: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string denoting how or why an error was resolved.
    """

    field_in_error: typing.Optional[str] = pydantic.Field(default=None)
    """
    The field of the corresponding underlying ChargeCapture that has a field that is failing validations,
    is not present but marked as required, or otherwise in error.
    """

    claim_creation_id: ChargeCaptureClaimCreationId = pydantic.Field()
    """
    The ID of the ChargeCaptureClaimCreation associated with this Error.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        Gender,
        IndividualBase,
        IndividualBaseOptional,
        IndividualId,
        Patient,
        PatientBase,
        PatientClinicalTrialInfo,
        PatientClinicalTrialInfoCreate,
        PatientCreate,
        PatientNonInsurancePayerInfo,
        PatientNonInsurancePayerInfoCreate,
        PatientNonInsurancePayerInfoCreateOptional,
        PatientUpdate,
        PatientUpdateWithOptionalAddress,
        Subscriber,
        SubscriberBase,
        SubscriberBaseOptional,
        SubscriberCreate,
        SubscriberCreateOptional,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "Gender": ".types",
    "IndividualBase": ".types",
    "IndividualBaseOptional": ".types",
    "IndividualId": ".types",
    "Patient": ".types",
    "PatientBase": ".types",
    "PatientClinicalTrialInfo": ".types",
    "PatientClinicalTrialInfoCreate": ".types",
    "PatientCreate": ".types",
    "PatientNonInsurancePayerInfo": ".types",
    "PatientNonInsurancePayerInfoCreate": ".types",
    "PatientNonInsurancePayerInfoCreateOptional": ".types",
    "PatientUpdate": ".types",
    "PatientUpdateWithOptionalAddress": ".types",
    "Subscriber": ".types",
    "SubscriberBase": ".types",
    "SubscriberBaseOptional": ".types",
    "SubscriberCreate": ".types",
    "SubscriberCreateOptional": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Gender",
    "IndividualBase",
    "IndividualBaseOptional",
    "IndividualId",
    "Patient",
    "PatientBase",
    "PatientClinicalTrialInfo",
    "PatientClinicalTrialInfoCreate",
    "PatientCreate",
    "PatientNonInsurancePayerInfo",
    "PatientNonInsurancePayerInfoCreate",
    "PatientNonInsurancePayerInfoCreateOptional",
    "PatientUpdate",
    "PatientUpdateWithOptionalAddress",
    "Subscriber",
    "SubscriberBase",
    "SubscriberBaseOptional",
    "SubscriberCreate",
    "SubscriberCreateOptional",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....billing_notes.resources.v_2.types.billing_note import BillingNote
from .....claims.types.claim import Claim
from .....commons.types.charge_capture_claim_creation_id import ChargeCaptureClaimCreationId
from .....commons.types.encounter_id import EncounterId
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.link_url import LinkUrl
from .....commons.types.next_responsible_party import NextResponsibleParty
from .....commons.types.state import State
from .....commons.types.work_queue_id import WorkQueueId
from .....custom_schemas.resources.v_1.types.schema_instance import SchemaInstance
from .....diagnoses.types.diagnosis import Diagnosis
from .....encounter_providers.resources.v_2.types.encounter_provider import EncounterProvider
from .....guarantor.resources.v_1.types.guarantor import Guarantor
from .....individual.types.patient import Patient
from .....individual.types.subscriber import Subscriber
from .....patient_payments.resources.v_3.types.patient_payment import PatientPayment
from .....property_and_casualty.resources.v_1.types.property_casualty_patient_identifier import (
    PropertyCasualtyPatientIdentifier,
)
from .....related_causes.resources.v_1.types.related_causes_information import RelatedCausesInformation
from .....service_facility.types.encounter_service_facility import EncounterServiceFacility
from .....tags.types.tag import Tag
from .....x_12.resources.v_1.types.patient_discharge_status_code import PatientDischargeStatusCode
from .....x_12.resources.v_1.types.point_of_origin_for_admission_or_visit_code import (
    PointOfOriginForAdmissionOrVisitCode,
)
from .....x_12.resources.v_1.types.type_of_admission_or_visit_code import TypeOfAdmissionOrVisitCode
from .....x_12.resources.v_1.types.type_of_bill_composite import TypeOfBillComposite
from .claim_supplemental_information import ClaimSupplementalInformation
from .clinical_note_category import ClinicalNoteCategory
from .coding_attribution_type import CodingAttributionType
from .encounter_base import EncounterBase
from .encounter_owner_of_next_action_type import EncounterOwnerOfNextActionType
from .encounter_submission_expectation import EncounterSubmissionExpectation
from .encounter_submission_origin_type import EncounterSubmissionOriginType
from .epsdt_referral import EpsdtReferral
from .patient_history_category import PatientHistoryCategory
from .prior_authorization_number import PriorAuthorizationNumber
from .responsible_party_type import ResponsiblePartyType


class Encounter(EncounterBase):
    """
    Examples
    --------
    import datetime
    import uuid

    from candid.resources.billing_notes.resources.v_2 import BillingNote
    from candid.resources.claims import Claim, ClaimStatus
    from candid.resources.commons import (
        DateRangeOptionalEnd,
        FacilityTypeCode,
        InsuranceTypeCode,
        NextResponsibleParty,
        PatientRelationshipToInsuredCodeAll,
        PhoneNumber,
        PhoneNumberType,
        ProcedureModifier,
        ServiceLineUnits,
        SourceOfPaymentCode,
        State,
        StreetAddressLongZip,
        StreetAddressShortZip,
    )
    from candid.resources.custom_schemas.resources.v_1 import SchemaInstance
    from candid.resources.diagnoses import Diagnosis, DiagnosisTypeCode
    from candid.resources.encounter_providers.resources.v_2 import EncounterProvider
    from candid.resources.encounters.resources.v_4 import (
        BillableStatusType,
        ClinicalNote,
        ClinicalNoteCategory,
        CodingAttributionType,
        Encounter,
        EncounterOwnerOfNextActionType,
        EncounterSubmissionOriginType,
        IntakeFollowUp,
        IntakeQuestion,
        IntakeResponseAndFollowUps,
        Intervention,
        InterventionCategory,
        Lab,
        LabCodeType,
        Medication,
        NoteCategory,
        PatientHistoryCategory,
        PatientHistoryCategoryEnum,
        ResponsiblePartyType,
        ServiceAuthorizationExceptionCode,
        SynchronicityType,
    )
    from candid.resources.eras import Era
    from candid.resources.guarantor.resources.v_1 import Guarantor
    from candid.resources.individual import (
        Gender,
        Patient,
        PatientNonInsurancePayerInfo,
        Subscriber,
    )
    from candid.resources.insurance_cards.resources.v_2 import InsuranceCard
    from candid.resources.invoices import Invoice, InvoiceItem, InvoiceStatus
    from candid.resources.non_insurance_payers.resources.v_1 import (
        NonInsurancePayer,
    )
    from candid.resources.patient_payments.resources.v_3 import (
        PatientPayment,
        PatientPaymentSource,
        PatientPaymentStatus,
    )
    from candid.resources.service_facility import EncounterServiceFacility
    from candid.resources.service_lines.resources.v_2 import (
        DenialReasonContent,
        ServiceLine,
        ServiceLineAdjustment,
        ServiceLineDenialReason,
        ServiceLineEraData,
        TestResult,
        TestResultType,
    )
    from candid.resources.tags import Tag, TagColorEnum

    Encounter(
        encounter_id=uuid.UUID(
            "b2506e84-4470-4cba-8a68-6883367739e1",
        ),
        claims=[
            Claim(
                claim_id=uuid.UUID(
                    "dd9d7f82-37b5-449d-aa63-26925398335b",
                ),
                status=ClaimStatus.BILLER_RECEIVED,
                clearinghouse="Change Healthcare",
                clearinghouse_claim_id="5BA7C3AB-2BC2-496C-8B10-6CAC73D0729D",
                payer_claim_id="9BB9F259-9756-4F16-8F53-9DBB9F7EB1BB",
                service_lines=[
                    ServiceLine(
                        created_at=datetime.datetime.fromisoformat(
                            "2023-01-01 00:00:00+00:00",
                        ),
                        modifiers=[ProcedureModifier.TWENTY_TWO],
                        charge_amount_cents=10000,
                        allowed_amount_cents=8000,
                        insurance_balance_cents=0,
                        patient_balance_cents=2000,
                        paid_amount_cents=8000,
                        patient_responsibility_cents=2000,
                        diagnosis_id_zero=uuid.UUID(
                            "4ac84bcd-12f5-4f86-a57b-e06749127c98",
                        ),
                        diagnosis_id_one=uuid.UUID(
                            "eea5ca5a-8b43-45fd-8cbd-c6cc1103e759",
                        ),
                        diagnosis_id_two=uuid.UUID(
                            "5c4aa029-2db9-4202-916e-e93c708f65ff",
                        ),
                        diagnosis_id_three=uuid.UUID(
                            "81795126-a3ac-443c-b47e-7259a16ab4a2",
                        ),
                        service_line_era_data=ServiceLineEraData(
                            service_line_adjustments=[
                                ServiceLineAdjustment(
                                    created_at=datetime.datetime.fromisoformat(
                                        "2023-01-01 00:00:00+00:00",
                                    ),
                                    adjustment_group_code="CO",
                                    adjustment_reason_code="CO",
                                    adjustment_amount_cents=1000,
                                    adjustment_note="test_note",
                                )
                            ],
                            remittance_advice_remark_codes=["N362"],
                        ),
                        service_line_manual_adjustments=[
                            ServiceLineAdjustment(
                                created_at=datetime.datetime.fromisoformat(
                                    "2023-01-01 00:00:00+00:00",
                                ),
                                adjustment_group_code="CO",
                                adjustment_reason_code="CO",
                                adjustment_amount_cents=1000,
                                adjustment_note="test_note",
                            )
                        ],
                        related_invoices=[
                            Invoice(
                                id=uuid.UUID(
                                    "901be2f1-41bc-456e-9987-4fe2f84f9d75",
                                ),
                                created_at=datetime.datetime.fromisoformat(
                                    "2023-01-01 00:00:00+00:00",
                                ),
                                updated_at=datetime.datetime.fromisoformat(
                                    "2023-01-01 00:00:00+00:00",
                                ),
                                organzation_id=uuid.UUID(
                                    "f13f73d4-4344-46ea-9d93-33bcffbb9f36",
                                ),
                                source_id="9B626577-8808-4F28-9ED1-F0DFF0D49BBC",
                                source_customer_id="624D1972-8C69-4C2F-AEFA-10856F734DB3",
                                patient_external_id="10FED4D6-4C5A-48DF-838A-EEF45A74788D",
                                note="test_note",
                                due_date="2023-10-10",
                                status=InvoiceStatus.DRAFT,
                                url="https://example.com",
                                customer_invoice_url="https://example.com",
                                items=[
                                    InvoiceItem(
                                        service_line_id=uuid.UUID(
                                            "ced00f23-6e68-4678-9dbc-f5aa2969a565",
                                        ),
                                        amount_cents=500,
                                    )
                                ],
                            )
                        ],
                        denial_reason=ServiceLineDenialReason(
                            reason=DenialReasonContent.AUTHORIZATION_REQUIRED,
                        ),
                        place_of_service_code=FacilityTypeCode.PHARMACY,
                        service_line_id=uuid.UUID(
                            "ced00f23-6e68-4678-9dbc-f5aa2969a565",
                        ),
                        procedure_code="99213",
                        quantity="1",
                        units=ServiceLineUnits.MJ,
                        claim_id=uuid.UUID(
                            "026a1fb8-748e-4859-a2d7-3ea9e07d25ae",
                        ),
                        date_of_service_range=DateRangeOptionalEnd(
                            start_date="2023-01-01",
                            end_date="2023-01-03",
                        ),
                        date_of_service=datetime.date.fromisoformat(
                            "2023-01-01",
                        ),
                        end_date_of_service=datetime.date.fromisoformat(
                            "2023-01-03",
                        ),
                        test_results=[
                            TestResult(
                                result_type=TestResultType.HEMOGLOBIN,
                                value=51.0,
                            ),
                            TestResult(
                                result_type=TestResultType.LDL,
                                value=110.0,
                            ),
                        ],
                    )
                ],
                eras=[
                    Era(
                        era_id=uuid.UUID(
                            "4d844ef1-2253-43cd-a4f1-6db7e65cb54b",
                        ),
                        check_number="CHK12345",
                        check_date="2023-10-12",
                    )
                ],
            )
        ],
        patient=Patient(
            individual_id=uuid.UUID(
                "93ddbebf-4956-4482-9a6c-21499b7e4e5d",
            ),
            phone_numbers=[
                PhoneNumber(
                    number="1234567890",
                    type=PhoneNumberType.HOME,
                )
            ],
            non_insurance_payers=[
                NonInsurancePayer(
                    non_insurance_payer_id=uuid.UUID(
                        "eb7623ab-d5bc-4b25-b257-2b8fcec578de",
                    ),
                    name="Sunrise Foundation",
                    category="Foundation",
                    description="Sunrise Foundation is a non-profit organization that provides financial assistance to patients in need.",
                    enabled=True,
                    address=StreetAddressShortZip(
                        address_1="123 Main St",
                        city="San Francisco",
                        state=State.CA,
                        zip_code="94105",
                    ),
                    clinical_trials=[],
                )
            ],
            non_insurance_payers_info=[
                PatientNonInsurancePayerInfo(
                    non_insurance_payer=NonInsurancePayer(
                        non_insurance_payer_id=uuid.UUID(
                            "eb7623ab-d5bc-4b25-b257-2b8fcec578de",
                        ),
                        name="Sunrise Foundation",
                        category="Foundation",
                        description="Sunrise Foundation is a non-profit organization that provides financial assistance to patients in need.",
                        enabled=True,
                        address=StreetAddressShortZip(
                            address_1="123 Main St",
                            city="San Francisco",
                            state=State.CA,
                            zip_code="94105",
                        ),
                        clinical_trials=[],
                    ),
                    member_id="123456789",
                    clinical_trial_info=[],
                )
            ],
            phone_consent=True,
            email="johndoe@joincandidhealth.com",
            email_consent=True,
            auto_charge_consent=True,
            external_id="49460F77-6456-41F1-AC6D-0AED08614D39",
            date_of_birth=datetime.date.fromisoformat(
                "2000-01-01",
            ),
            address=StreetAddressShortZip(
                address_1="123 Main St",
                address_2="Apt 1",
                city="New York",
                state=State.NY,
                zip_code="10001",
                zip_plus_four_code="1234",
            ),
            first_name="John",
            last_name="Doe",
            gender=Gender.MALE,
        ),
        guarantor=Guarantor(
            guarantor_id=uuid.UUID(
                "8bbdbe63-58d3-4d40-98c9-40403c050977",
            ),
            phone_numbers=[
                PhoneNumber(
                    number="1234567890",
                    type=PhoneNumberType.HOME,
                )
            ],
            phone_consent=True,
            email="johndoe@joincandidhealth.com",
            email_consent=True,
            auto_charge_consent=True,
            first_name="John",
            last_name="Doe",
            external_id="49460F77-6456-41F1-AC6D-0AED08614D39",
            date_of_birth=datetime.date.fromisoformat(
                "2000-01-01",
            ),
            address=StreetAddressShortZip(
                address_1="123 Main St",
                address_2="Apt 1",
                city="New York",
                state=State.NY,
                zip_code="10001",
                zip_plus_four_code="1234",
            ),
        ),
        billing_provider=EncounterProvider(
            provider_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            address=StreetAddressLongZip(
                address_1="123 Main St",
                address_2="Apt 1",
                city="New York",
                state=State.NY,
                zip_code="10001",
                zip_plus_four_code="1234",
            ),
            tax_id="123456789",
            npi="1234567890",
            taxonomy_code="207Q00000X",
            first_name="John",
            last_name="Doe",
            organization_name="Organization Name",
        ),
        rendering_provider=EncounterProvider(
            provider_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            address=StreetAddressLongZip(
                address_1="123 Main St",
                address_2="Apt 1",
                city="New York",
                state=State.NY,
                zip_code="10001",
                zip_plus_four_code="1234",
            ),
            tax_id="123456789",
            npi="1234567890",
            taxonomy_code="207Q00000X",
            first_name="John",
            last_name="Doe",
            organization_name="Organization Name",
        ),
        referring_provider=EncounterProvider(
            provider_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            address=StreetAddressLongZip(
                address_1="123 Main St",
                address_2="Apt 1",
                city="New York",
                state=State.NY,
                zip_code="10001",
                zip_plus_four_code="1234",
            ),
            tax_id="123456789",
            npi="1234567890",
            taxonomy_code="207Q00000X",
            first_name="John",
            last_name="Doe",
            organization_name="Organization Name",
        ),
        service_facility=EncounterServiceFacility(
            service_facility_id=uuid.UUID(
                "2861487b-232c-4ded-a874-616a5db0688f",
            ),
            organization_name="Test Organization",
            address=StreetAddressLongZip(
                address_1="123 Main St",
                address_2="Apt 1",
                city="New York",
                state=State.NY,
                zip_code="10001",
                zip_plus_four_code="1234",
            ),
        ),
        subscriber_primary=Subscriber(
            individual_id=uuid.UUID(
                "797348a9-e7e8-4e59-8628-95390d079c0b",
            ),
            insurance_card=InsuranceCard(
                insurance_card_id=uuid.UUID(
                    "ca5b7711-4419-4161-9b7c-3494ac40c8d4",
                ),
                member_id="E85313B4-0FFC-4119-8042-8161A4ECFF0A",
                payer_name="John Doe",
                payer_id="836DDAA6-863F-4020-ACCA-205A689F0002",
                rx_bin="610014",
                rx_pcn="MEDDPRIME",
                image_url_front="https://s3.amazonaws.com/front.jpg",
                image_url_back="https://s3.amazonaws.com/back.jpg",
                group_number="ABC12345",
                plan_name="Silver PPO Plan",
                plan_type=SourceOfPaymentCode.SELF_PAY,
                insurance_type=InsuranceTypeCode.C_12,
            ),
            patient_relationship_to_subscriber_code=PatientRelationshipToInsuredCodeAll.SPOUSE,
            date_of_birth=datetime.date.fromisoformat(
                "2000-01-01",
            ),
            address=StreetAddressShortZip(
                address_1="123 Main St",
                address_2="Apt 1",
                city="New York",
                state=State.NY,
                zip_code="10001",
                zip_plus_four_code="1234",
            ),
            first_name="John",
            last_name="Doe",
            gender=Gender.MALE,
        ),
        subscriber_secondary=Subscriber(
            individual_id=uuid.UUID(
                "797348a9-e7e8-4e59-8628-95390d079c0b",
            ),
            insurance_card=InsuranceCard(
                insurance_card_id=uuid.UUID(
                    "ca5b7711-4419-4161-9b7c-3494ac40c8d4",
                ),
                member_id="E85313B4-0FFC-4119-8042-8161A4ECFF0A",
                payer_name="John Doe",
                payer_id="836DDAA6-863F-4020-ACCA-205A689F0002",
                rx_bin="610014",
                rx_pcn="MEDDPRIME",
                image_url_front="https://s3.amazonaws.com/front.jpg",
                image_url_back="https://s3.amazonaws.com/back.jpg",
                group_number="ABC12345",
                plan_name="Silver PPO Plan",
                plan_type=SourceOfPaymentCode.SELF_PAY,
                insurance_type=InsuranceTypeCode.C_12,
            ),
            patient_relationship_to_subscriber_code=PatientRelationshipToInsuredCodeAll.SPOUSE,
            date_of_birth=datetime.date.fromisoformat(
                "2000-01-01",
            ),
            address=StreetAddressShortZip(
                address_1="123 Main St",
                address_2="Apt 1",
                city="New York",
                state=State.NY,
                zip_code="10001",
                zip_plus_four_code="1234",
            ),
            first_name="John",
            last_name="Doe",
            gender=Gender.MALE,
        ),
        url="https://example.com",
        diagnoses=[
            Diagnosis(
                diagnosis_id=uuid.UUID(
                    "5c770e00-4bbf-42af-a73f-99c5e91fc0db",
                ),
                created_at=datetime.datetime.fromisoformat(
                    "2023-01-01 00:00:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2023-01-01 00:00:00+00:00",
                ),
                encounter_id=uuid.UUID(
                    "3f63985b-51a4-4dd4-9418-7d50b2520792",
                ),
                name="John Doe",
                code_type=DiagnosisTypeCode.ABF,
                code="I10",
            )
        ],
        clinical_notes=[
            ClinicalNoteCategory(
                category=NoteCategory.CLINICAL,
                notes=["Patient complained of mild chest pain."],
                notes_structured=[
                    ClinicalNote(
                        text="Mild chest pain since morning.",
                        author_name="John Doe",
                        author_npi="1234567890",
                        timestamp=datetime.datetime.fromisoformat(
                            "2023-01-01 00:00:00+00:00",
                        ),
                    )
                ],
            )
        ],
        billing_notes=[
            BillingNote(
                billing_note_id=uuid.UUID(
                    "99882eea-936f-4e71-bc4f-520e4d14e3e2",
                ),
                encounter_id=uuid.UUID(
                    "8bcfb6a8-2876-4111-9e3f-602b541fcf62",
                ),
                created_at=datetime.datetime.fromisoformat(
                    "2023-01-01 00:00:00+00:00",
                ),
                author_auth_0_id="F0DE3BF9-F9A1-4FA7-BF6B-28C0B46BADD8",
                author_name="John Doe",
                text="Patient was billed for an MRI.",
            )
        ],
        place_of_service_code=FacilityTypeCode.PHARMACY,
        place_of_service_code_as_submitted=FacilityTypeCode.PHARMACY,
        patient_histories=[
            PatientHistoryCategory(
                category=PatientHistoryCategoryEnum.PRESENT_ILLNESS,
                questions=[
                    IntakeQuestion(
                        id="6E7FBCE4-A8EA-46D0-A8D8-FF83CA3BB176",
                        text="Do you have any allergies?",
                        responses=[
                            IntakeResponseAndFollowUps(
                                response="No allergies",
                                follow_ups=[
                                    IntakeFollowUp(
                                        id="4F3D57F9-AC94-49D6-87E4-E804B709917A",
                                        text="Do you have any allergies?",
                                        response="No allergies",
                                    )
                                ],
                            )
                        ],
                    )
                ],
            )
        ],
        patient_payments=[
            PatientPayment(
                patient_payment_id="CF237BE1-E793-4BBF-8958-61D5179D1D0D",
                organization_id=uuid.UUID(
                    "0788ca2a-b20d-4b8e-b8d4-07fa0b3b4907",
                ),
                source_internal_id="D1A76039-D5C5-4323-A2FC-B7C8B6AEF6A4",
                source=PatientPaymentSource.MANUAL_ENTRY,
                amount_cents=2000,
                payment_timestamp=datetime.datetime.fromisoformat(
                    "2023-01-01 00:00:00+00:00",
                ),
                status=PatientPaymentStatus.PENDING,
                payment_name="John Doe",
                payment_note="test payment note",
                patient_external_id="B7437260-D6B4-48CF-B9D7-753C09F34E76",
                encounter_external_id="0F26B9C3-199F-4CBB-A166-B87EA7C631BB",
                service_line_id=uuid.UUID(
                    "b557dc86-c629-478c-850a-02d45ac11783",
                ),
            )
        ],
        tags=[
            Tag(
                creator_id="00EB5A46-35C6-441B-9751-AF307AEF5888",
                tag_id="void-claim-submitted",
                description="to indicate claims where a void claim has been submitted",
                color=TagColorEnum.BLACK,
            )
        ],
        coding_attribution=CodingAttributionType.CANDID,
        work_queue_id="12",
        work_queue_membership_activated_at=datetime.datetime.fromisoformat(
            "2023-01-01 00:00:00+00:00",
        ),
        owner_of_next_action=EncounterOwnerOfNextActionType.CANDID,
        submission_origin=EncounterSubmissionOriginType.CANDID,
        external_id="5C21490F-A9C0-45F4-B5DB-136E3AEC617A",
        date_of_service=datetime.date.fromisoformat(
            "2023-01-01",
        ),
        end_date_of_service=datetime.date.fromisoformat(
            "2023-01-05",
        ),
        prior_authorization_number="PA1234567",
        patient_authorized_release=True,
        benefits_assigned_to_provider=True,
        provider_accepts_assignment=True,
        appointment_type="Routine Checkup",
        existing_medications=[
            Medication(
                name="Lisinopril",
                rx_cui="860975",
                dosage="10mg",
                dosage_form="Tablet",
                frequency="Once Daily",
                as_needed=True,
            )
        ],
        interventions=[
            Intervention(
                name="Physical Therapy Session",
                category=InterventionCategory.LIFESTYLE,
                description="A session focused on improving muscular strength, flexibility, and range of motion post-injury.",
                medication=Medication(
                    name="Lisinopril",
                    rx_cui="860975",
                    dosage="10mg",
                    dosage_form="Tablet",
                    frequency="Once Daily",
                    as_needed=True,
                ),
                labs=[
                    Lab(
                        name="Genetic Health Labs",
                        code="GH12345",
                        code_type=LabCodeType.QUEST,
                    )
                ],
            )
        ],
        pay_to_address=StreetAddressLongZip(
            address_1="123 Main St",
            address_2="Apt 1",
            city="New York",
            state=State.NY,
            zip_code="10001",
            zip_plus_four_code="1234",
        ),
        synchronicity=SynchronicityType.SYNCHRONOUS,
        billable_status=BillableStatusType.BILLABLE,
        responsible_party=ResponsiblePartyType.INSURANCE_PAY,
        service_authorization_exception_code=ServiceAuthorizationExceptionCode.C_1,
        admission_date=datetime.date.fromisoformat(
            "2023-01-01",
        ),
        discharge_date=datetime.date.fromisoformat(
            "2023-01-05",
        ),
        onset_of_current_illness_or_symptom_date=datetime.date.fromisoformat(
            "2023-01-01",
        ),
        schema_instances=[
            SchemaInstance(
                schema_id=uuid.UUID(
                    "ec096b13-f80a-471d-aaeb-54b021c9d582",
                ),
                content={
                    "provider_category": "internist",
                    "is_urgent_care": True,
                    "bmi": 24.2,
                    "age": 38,
                },
            )
        ],
        created_at=datetime.datetime.fromisoformat(
            "2023-01-01 00:00:00+00:00",
        ),
        next_responsible_party=NextResponsibleParty.PRIMARY,
    )
    """

    accident_state_or_province_code: typing.Optional[State] = pydantic.Field(default=None)
    """
    837i-REF1000 -- an optional state indicating where an accident related to the encounter occurred.
    """

    claim_creation_id: typing.Optional[ChargeCaptureClaimCreationId] = pydantic.Field(default=None)
    """
    If the encounter was created from ingested charge captures, this is the associated Charge Capture Claim Creation Id.
    """

    patient_control_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    A patient control number (PCN) is a unique identifier assigned to a patient within a healthcare system or facility.
    It's used to track and manage a patient's medical records, treatments, and other healthcare-related information.
    """

    encounter_id: EncounterId
    claims: typing.List[Claim]
    patient: Patient = pydantic.Field()
    """
    Contains the identification information of the individual receiving medical services.
    """

    guarantor: typing.Optional[Guarantor] = pydantic.Field(default=None)
    """
    Personal and contact info for the guarantor of the patient responsibility.
    """

    billing_provider: EncounterProvider = pydantic.Field()
    """
    The billing provider is the provider or business entity submitting the claim. Billing provider may be, but is not necessarily, the same person/NPI as the rendering provider. From a payer's perspective, this represents the person or entity being reimbursed. When a contract exists with the target payer, the billing provider should be the entity contracted with the payer. In some circumstances, this will be an individual provider. In that case, submit that provider's NPI and the tax ID (TIN) that the provider gave to the payer during contracting. In other cases, the billing entity will be a medical group. If so, submit the group NPI and the group's tax ID. Box 33 on the CMS-1500 claim form or Form Locator 1 on a UB-04 claim form.
    """

    rendering_provider: EncounterProvider = pydantic.Field()
    """
    The rendering provider is the practitioner -- physician, nurse practitioner, etc. -- performing the service.
    For telehealth services, the rendering provider performs the visit, asynchronous communication, or other service. The rendering provider address should generally be the same as the service facility address.
    """

    attending_provider: typing.Optional[EncounterProvider] = pydantic.Field(default=None)
    """
    837i NM1 2500 variant for Loop ID-2310.  Used to indicate the individual whom has overall responsibility for the patient in institutional claims processing.
    """

    admission_hour: typing.Optional[int] = pydantic.Field(default=None)
    """
    837i Loop 2300 DTP-03
    Extension of the admission date with hour (0-23) details.
    Required for institutional submission.
    """

    admission_type_code: typing.Optional[TypeOfAdmissionOrVisitCode] = pydantic.Field(default=None)
    """
    837i Loop 2300 CL1-01 Code used to indicate the priority of an admission or visit. Equivalent to Form Locator 14 Priority of Admission on a UB-04 claim, not used on CMS-1500 claim forms.
    """

    admission_source_code: typing.Optional[PointOfOriginForAdmissionOrVisitCode] = pydantic.Field(default=None)
    """
    837i Loop 2300 CLI1-02 Code used to indicate the conditions under which an admission occurs. Equivalent to Form Locator 15 Point of Origin on a UB-04 claim, not used on CMS-1500 claim forms.
    """

    discharge_hour: typing.Optional[int] = pydantic.Field(default=None)
    """
    837i Loop 2300 DTP-03
    Extension of the discharge date with hour (0-23) details.
    """

    discharge_status: typing.Optional[PatientDischargeStatusCode] = pydantic.Field(default=None)
    """
    837i CL1-03 or Form Locator 17 on a UB-04 claim form. This is a required field on UB-04 claims.
    Code indicating patient status as of the "statement covers through date".
    """

    operating_provider: typing.Optional[EncounterProvider] = pydantic.Field(default=None)
    """
    837i NM1 2500 variant for Loop ID-2310.  Used to indicate the individual whom has primary responsibility for surgical procedures in institutional claims processing.
    """

    other_operating_provider: typing.Optional[EncounterProvider] = pydantic.Field(default=None)
    """
    837i NM1 2500 variant for Loop ID-2310.  Used to indicate the individual whom has secondary responsibility for surgical procedures in institutional claims processing.  Only used when operating_provider is also set.
    """

    related_causes_information: typing.Optional[RelatedCausesInformation] = None
    property_casualty_claim_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    837p Loop2010 REF02, CMS1500 Box 11b
    """

    accident_date: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP*439, CMS1500 Box 15
    """

    property_casualty_patient_identifier: typing.Optional[PropertyCasualtyPatientIdentifier] = pydantic.Field(
        default=None
    )
    """
    Patient identifier for Property and Casualty claims.
    837p Loop2010CA
    """

    submission_expectation: typing.Optional[EncounterSubmissionExpectation] = pydantic.Field(default=None)
    """
    Describes the currently expected target form for this encounter.  This effects what validations and queues the form is processed under.  When this value is not set, it should be assumed to be TARGET_PROFESSIONAL.
    """

    type_of_bill: typing.Optional[TypeOfBillComposite] = pydantic.Field(default=None)
    """
    Four digit code used in institutional forms to indicate the type of bill (e.g., hospital inpatient, hospital outpatient). First digit is a leading 0, followed by the type_of_facility, type_of_care, then frequency_code. Professional forms are not required to submit this attribute. You may send the 4 digit code via raw_code, or each individual digit separately via composite_codes.
    """

    referring_provider: typing.Optional[EncounterProvider] = None
    initial_referring_provider: typing.Optional[EncounterProvider] = None
    supervising_provider: typing.Optional[EncounterProvider] = None
    service_facility: EncounterServiceFacility = pydantic.Field()
    """
    Encounter Service facility is typically the location a medical service was rendered, such as a provider office or hospital. For telehealth, service facility can represent the provider's location when the service was delivered (e.g., home), or the location where an in-person visit would have taken place, whichever is easier to identify. If the provider is in-network, service facility may be defined in payer contracts. Box 32 on the CMS-1500 claim form. There is no equivalent on the paper UB-04 claim form, but this field is equivalent to Loop 2310E Service Facility Location details on an 837i form, and is used when this is different to the entity identified as the Billing Provider. Note that for an in-network claim to be successfully adjudicated, the service facility address listed
    """

    subscriber_primary: typing.Optional[Subscriber] = pydantic.Field(default=None)
    """
    Subscriber_primary is required when responsible_party is INSURANCE_PAY (i.e. when the claim should be billed to insurance).
    These are not required fields when responsible_party is SELF_PAY (i.e. when the claim should be billed to the patient).
    However, if you collect this for patients, even self-pay, we recommend including it when sending encounters to Candid.
    Note: Cash Pay is no longer a valid payer_id in v4, please use responsible party to define self-pay claims.
    """

    subscriber_secondary: typing.Optional[Subscriber] = pydantic.Field(default=None)
    """
    Contains details of the secondary insurance subscriber.
    """

    subscriber_tertiary: typing.Optional[Subscriber] = pydantic.Field(default=None)
    """
    Contains details of the tertiary insurance subscriber.
    """

    prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = pydantic.Field(default=None)
    """
    Box 23 on the CMS-1500 claim form or Form Locator 63 on a UB-04 claim form.
    """

    responsible_party: ResponsiblePartyType = pydantic.Field()
    """
    Defines the party to be billed with the initial balance owed on the claim.
    """

    url: LinkUrl = pydantic.Field()
    """
    URL that links directly to the claim created in Candid.
    """

    diagnoses: typing.List[Diagnosis] = pydantic.Field()
    """
    Contains the primary and other diagnosis health care code information objects associated with this encounter. For professional claims, these diagnoses correspond with those that are set on service lines directly, where as for institutional claims these are only associated directly with the claim itself.  See also Health Care Code Information objects and corresponding apis.
    """

    clinical_notes: typing.List[ClinicalNoteCategory] = pydantic.Field()
    """
    Holds a collection of clinical observations made by healthcare providers during patient encounters. Please note that medical records for appeals should be sent using the Encounter Attachments API.
    """

    billing_notes: typing.Optional[typing.List[BillingNote]] = pydantic.Field(default=None)
    """
    Spot to store misc, human-readable, notes about this encounter to be
    used in the billing process.
    """

    place_of_service_code: typing.Optional[FacilityTypeCode] = pydantic.Field(default=None)
    """
    Box 24B on the CMS-1500 claim form. 837p Loop2300, CLM-05-1. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).
    """

    place_of_service_code_as_submitted: typing.Optional[FacilityTypeCode] = pydantic.Field(default=None)
    """
    Box 24B on the CMS-1500 claim form. 837p Loop2300, CLM-05-1. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).
    """

    patient_histories: typing.List[PatientHistoryCategory]
    patient_payments: typing.List[PatientPayment]
    tags: typing.List[Tag]
    coding_attribution: typing.Optional[CodingAttributionType] = pydantic.Field(default=None)
    """
    The entity that performed the coding of medical services for the claim.
    """

    work_queue_id: typing.Optional[WorkQueueId] = None
    work_queue_membership_activated_at: typing.Optional[dt.datetime] = None
    owner_of_next_action: EncounterOwnerOfNextActionType = pydantic.Field()
    """
    The party who is responsible for taking the next action on an Encounter, as defined by ownership of open Tasks.
    """

    submission_origin: EncounterSubmissionOriginType = pydantic.Field()
    """
    The party who originally submitted the Claim.
    For Claims originating in Candid, this will be EncounterSubmissionOriginType.CANDID.
    For Encounters created with an external_claim_submission object, this will be EncounterSubmissionOriginType.EXTERNAL.
    """

    schema_instances: typing.List[SchemaInstance] = pydantic.Field()
    """
    Key-value pairs that must adhere to a schema created via the Custom Schema API. Multiple schema
    instances cannot be created for the same schema on an encounter.
    """

    referral_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Refers to REF*9F on the 837p. Value cannot be greater than 50 characters.
    """

    epsdt_referral: typing.Optional[EpsdtReferral] = pydantic.Field(default=None)
    """
    Refers to Box 24H on the CMS1500 form and Loop 2300 CRC - EPSDT Referral on the 837P and 837i form
    """

    claim_supplemental_information: typing.Optional[typing.List[ClaimSupplementalInformation]] = pydantic.Field(
        default=None
    )
    """
    Refers to Loop 2300 - Segment PWK on the 837P and 837i form. No more than 10 entries are permitted.
    """

    secondary_payer_carrier_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    When Medicaid is billed as the secondary payer the Carrier Code is used to identify the primary payer. This is required for certain states.
    """

    last_submitted_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The date and time the encounter was last submitted to a payer.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    The date and time the encounter was created.
    """

    next_responsible_party: typing.Optional[NextResponsibleParty] = pydantic.Field(default=None)
    """
    The party (payer, patient, etc.) responsible for the remainder of the balance on the claim.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.patient_id import PatientId
from .....coverages.resources.v_1.types.coverage_status import CoverageStatus
from .....patients.resources.v_1.types.mutable_patient_with_mrn import MutablePatientWithMrn
from .appointment_status import AppointmentStatus


class Visit(UniversalBaseModel):
    """
    A visit is a collection of appointments that occur on the same day.
    """

    patient_id: PatientId
    patient: MutablePatientWithMrn
    start_time: dt.datetime
    status: AppointmentStatus
    primary_coverage_status: typing.Optional[CoverageStatus] = None
    secondary_coverage_status: typing.Optional[CoverageStatus] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import uuid
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....charge_capture.resources.v_1.types.charge_capture_status import ChargeCaptureStatus
from ....claims.types.claim_status import ClaimStatus
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.types.charge_capture_claim_creation_id import ChargeCaptureClaimCreationId
from ....commons.types.encounter_id import EncounterId
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.sort_direction import SortDirection
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from .types.charge_capture_claim_creation import ChargeCaptureClaimCreation
from .types.charge_capture_claim_creation_page import ChargeCaptureClaimCreationPage
from .types.charge_capture_claim_creation_sort_field import ChargeCaptureClaimCreationSortField
from .types.charge_capture_claim_creation_status import ChargeCaptureClaimCreationStatus
from .types.charge_capture_claim_creation_summary import ChargeCaptureClaimCreationSummary

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        charge_capture_claim_creation_id: ChargeCaptureClaimCreationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ChargeCaptureClaimCreation]:
        """
        Parameters
        ----------
        charge_capture_claim_creation_id : ChargeCaptureClaimCreationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ChargeCaptureClaimCreation]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/charge_capture_claim_creation/v1/{jsonable_encoder(charge_capture_claim_creation_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ChargeCaptureClaimCreation,
                parse_obj_as(
                    type_=ChargeCaptureClaimCreation,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_summary(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ChargeCaptureClaimCreationSummary]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ChargeCaptureClaimCreationSummary]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/charge_capture_claim_creation/v1/all/summary",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ChargeCaptureClaimCreationSummary,
                parse_obj_as(
                    type_=ChargeCaptureClaimCreationSummary,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def resolve_charge_creation_error(
        self,
        charge_capture_bundle_error_id: uuid.UUID,
        *,
        resolved_by: typing.Optional[str] = OMIT,
        resolution_reason: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Parameters
        ----------
        charge_capture_bundle_error_id : uuid.UUID

        resolved_by : typing.Optional[str]
            A string, denoting who resolved the error for audit trail purposes.

        resolution_reason : typing.Optional[str]
            A string denoting why or how the error was dealt with for audit trail purposes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/charge_capture_claim_creation/v1/error/{jsonable_encoder(charge_capture_bundle_error_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "resolved_by": resolved_by,
                "resolution_reason": resolution_reason,
            },
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        sort: typing.Optional[ChargeCaptureClaimCreationSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        patient_external_id: typing.Optional[str] = None,
        claim_creation_status: typing.Optional[ChargeCaptureClaimCreationStatus] = None,
        charge_status: typing.Optional[ChargeCaptureStatus] = None,
        charge_external_id: typing.Optional[str] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        claim_ids: typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]] = None,
        claim_creation_ids: typing.Optional[
            typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]
        ] = None,
        billing_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_facility_name: typing.Optional[str] = None,
        primary_payer_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        claim_creation_category: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tags: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        primary_payer_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        patient_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        has_charge_capture_updates: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ChargeCaptureClaimCreationPage]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        sort : typing.Optional[ChargeCaptureClaimCreationSortField]
            Defaults to created_at

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        claim_creation_status : typing.Optional[ChargeCaptureClaimCreationStatus]
            the status of the charge capture Claim Creation, refers to whether it was able to create a claim.

        charge_status : typing.Optional[ChargeCaptureStatus]
            the status of the charge captures

        charge_external_id : typing.Optional[str]
            A client-specified unique ID to associate with this encounter;
            for example, your internal encounter ID or a Dr. Chrono encounter ID.
            This field should not contain PHI.

        date_of_service_min : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        date_of_service_max : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        claim_ids : typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]]
            A list of claim IDs to filter by. This will return all charge capture claim_creations that have a resulting claim with one of the IDs in this list.

        claim_creation_ids : typing.Optional[typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]]
            A list of Claim Creation IDs to filter by.

        billing_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of billing provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        service_facility_name : typing.Optional[str]
            A string to filter by. This will return all charge capture claim_creations which include one or more charges with this service facility name.

        primary_payer_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer IDs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the primary payer IDs in this list.

        rendering_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        rendering_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider names to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        supervising_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        supervising_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider names to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        claim_status : typing.Optional[ClaimStatus]
            the status of the claim to filter by created from charge capture bundle.

        claim_creation_category : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of claim creation categories to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        tags : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of tags to filter by. This will return all charge captures with one of the tags.

        primary_payer_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer names to filter by. This will return all charge captures with one of the names.

        patient_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of patient names to filter by. This will return all charge captures with one of the names.

        has_charge_capture_updates : typing.Optional[bool]
            If true, only return claim_creations that have charge captures that have been updated since the Claim Creation has had a status of BILLED. See the updates property on ChargeCapture for more details.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ChargeCaptureClaimCreationPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/charge_capture_claim_creation/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "sort": sort,
                "sort_direction": sort_direction,
                "page_token": page_token,
                "patient_external_id": patient_external_id,
                "claim_creation_status": claim_creation_status,
                "charge_status": charge_status,
                "charge_external_id": charge_external_id,
                "date_of_service_min": str(date_of_service_min) if date_of_service_min is not None else None,
                "date_of_service_max": str(date_of_service_max) if date_of_service_max is not None else None,
                "claim_ids": claim_ids,
                "claim_creation_ids": claim_creation_ids,
                "billing_provider_npis": billing_provider_npis,
                "service_facility_name": service_facility_name,
                "primary_payer_ids": primary_payer_ids,
                "rendering_provider_npis": rendering_provider_npis,
                "rendering_provider_names": rendering_provider_names,
                "supervising_provider_npis": supervising_provider_npis,
                "supervising_provider_names": supervising_provider_names,
                "claim_status": claim_status,
                "claim_creation_category": claim_creation_category,
                "tags": tags,
                "primary_payer_names": primary_payer_names,
                "patient_names": patient_names,
                "has_charge_capture_updates": has_charge_capture_updates,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ChargeCaptureClaimCreationPage,
                parse_obj_as(
                    type_=ChargeCaptureClaimCreationPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        charge_capture_claim_creation_id: ChargeCaptureClaimCreationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ChargeCaptureClaimCreation]:
        """
        Parameters
        ----------
        charge_capture_claim_creation_id : ChargeCaptureClaimCreationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ChargeCaptureClaimCreation]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/charge_capture_claim_creation/v1/{jsonable_encoder(charge_capture_claim_creation_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ChargeCaptureClaimCreation,
                parse_obj_as(
                    type_=ChargeCaptureClaimCreation,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_summary(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ChargeCaptureClaimCreationSummary]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ChargeCaptureClaimCreationSummary]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/charge_capture_claim_creation/v1/all/summary",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ChargeCaptureClaimCreationSummary,
                parse_obj_as(
                    type_=ChargeCaptureClaimCreationSummary,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def resolve_charge_creation_error(
        self,
        charge_capture_bundle_error_id: uuid.UUID,
        *,
        resolved_by: typing.Optional[str] = OMIT,
        resolution_reason: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Parameters
        ----------
        charge_capture_bundle_error_id : uuid.UUID

        resolved_by : typing.Optional[str]
            A string, denoting who resolved the error for audit trail purposes.

        resolution_reason : typing.Optional[str]
            A string denoting why or how the error was dealt with for audit trail purposes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/charge_capture_claim_creation/v1/error/{jsonable_encoder(charge_capture_bundle_error_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "resolved_by": resolved_by,
                "resolution_reason": resolution_reason,
            },
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        sort: typing.Optional[ChargeCaptureClaimCreationSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        patient_external_id: typing.Optional[str] = None,
        claim_creation_status: typing.Optional[ChargeCaptureClaimCreationStatus] = None,
        charge_status: typing.Optional[ChargeCaptureStatus] = None,
        charge_external_id: typing.Optional[str] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        claim_ids: typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]] = None,
        claim_creation_ids: typing.Optional[
            typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]
        ] = None,
        billing_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_facility_name: typing.Optional[str] = None,
        primary_payer_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        claim_creation_category: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tags: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        primary_payer_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        patient_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        has_charge_capture_updates: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ChargeCaptureClaimCreationPage]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        sort : typing.Optional[ChargeCaptureClaimCreationSortField]
            Defaults to created_at

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        claim_creation_status : typing.Optional[ChargeCaptureClaimCreationStatus]
            the status of the charge capture Claim Creation, refers to whether it was able to create a claim.

        charge_status : typing.Optional[ChargeCaptureStatus]
            the status of the charge captures

        charge_external_id : typing.Optional[str]
            A client-specified unique ID to associate with this encounter;
            for example, your internal encounter ID or a Dr. Chrono encounter ID.
            This field should not contain PHI.

        date_of_service_min : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        date_of_service_max : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        claim_ids : typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]]
            A list of claim IDs to filter by. This will return all charge capture claim_creations that have a resulting claim with one of the IDs in this list.

        claim_creation_ids : typing.Optional[typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]]
            A list of Claim Creation IDs to filter by.

        billing_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of billing provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        service_facility_name : typing.Optional[str]
            A string to filter by. This will return all charge capture claim_creations which include one or more charges with this service facility name.

        primary_payer_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer IDs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the primary payer IDs in this list.

        rendering_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        rendering_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider names to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        supervising_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        supervising_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider names to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        claim_status : typing.Optional[ClaimStatus]
            the status of the claim to filter by created from charge capture bundle.

        claim_creation_category : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of claim creation categories to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        tags : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of tags to filter by. This will return all charge captures with one of the tags.

        primary_payer_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer names to filter by. This will return all charge captures with one of the names.

        patient_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of patient names to filter by. This will return all charge captures with one of the names.

        has_charge_capture_updates : typing.Optional[bool]
            If true, only return claim_creations that have charge captures that have been updated since the Claim Creation has had a status of BILLED. See the updates property on ChargeCapture for more details.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ChargeCaptureClaimCreationPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/charge_capture_claim_creation/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "sort": sort,
                "sort_direction": sort_direction,
                "page_token": page_token,
                "patient_external_id": patient_external_id,
                "claim_creation_status": claim_creation_status,
                "charge_status": charge_status,
                "charge_external_id": charge_external_id,
                "date_of_service_min": str(date_of_service_min) if date_of_service_min is not None else None,
                "date_of_service_max": str(date_of_service_max) if date_of_service_max is not None else None,
                "claim_ids": claim_ids,
                "claim_creation_ids": claim_creation_ids,
                "billing_provider_npis": billing_provider_npis,
                "service_facility_name": service_facility_name,
                "primary_payer_ids": primary_payer_ids,
                "rendering_provider_npis": rendering_provider_npis,
                "rendering_provider_names": rendering_provider_names,
                "supervising_provider_npis": supervising_provider_npis,
                "supervising_provider_names": supervising_provider_names,
                "claim_status": claim_status,
                "claim_creation_category": claim_creation_category,
                "tags": tags,
                "primary_payer_names": primary_payer_names,
                "patient_names": patient_names,
                "has_charge_capture_updates": has_charge_capture_updates,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ChargeCaptureClaimCreationPage,
                parse_obj_as(
                    type_=ChargeCaptureClaimCreationPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
